@@ -1,28 +1,31 @@
 import concurrent.futures
+import contextvars
+import copy
 import re
 
-from flask import copy_current_request_context
-from flask.globals import _app_ctx_stack, current_app
+from flask import copy_current_request_context, current_app, g
 
 from flask_executor.futures import FutureCollection, FutureProxy
 from flask_executor.helpers import InstanceProxy, str2bool
 
 
+def get_current_app_context():
+    try:
+        from flask.globals import _cv_app
+        return _cv_app.get(None)
+    except ImportError:
+        from flask.globals import _app_ctx_stack
+        return _app_ctx_stack.top
+
+
 def push_app_context(fn):
     app = current_app._get_current_object()
+    _g = copy.copy(g)
 
     def wrapper(*args, **kwargs):
         with app.app_context():
-            return fn(*args, **kwargs)
-
-    return wrapper
-
-
-def copy_current_app_context(fn):
-    app_context = _app_ctx_stack.top
-
-    def wrapper(*args, **kwargs):
-        with app_context:
+            ctx = get_current_app_context()
+            ctx.g = _g
             return fn(*args, **kwargs)
 
     return wrapper
@@ -93,7 +96,7 @@ class Executor(InstanceProxy, concurrent.futures._base.Executor):
             * :class:`concurrent.futures.ProcessPoolExecutor`
         """
         app.config.setdefault(self.EXECUTOR_TYPE, 'thread')
-        app.config.setdefault(self.EXECUTOR_PUSH_APP_CONTEXT, False)
+        app.config.setdefault(self.EXECUTOR_PUSH_APP_CONTEXT, True)
         futures_max_length = app.config.setdefault(self.EXECUTOR_FUTURES_MAX_LENGTH, None)
         propagate_exceptions = app.config.setdefault(self.EXECUTOR_PROPAGATE_EXCEPTIONS, False)
         if futures_max_length is not None:
@@ -122,8 +125,6 @@ class Executor(InstanceProxy, concurrent.futures._base.Executor):
             fn = copy_current_request_context(fn)
             if current_app.config[self.EXECUTOR_PUSH_APP_CONTEXT]:
                 fn = push_app_context(fn)
-            else:
-                fn = copy_current_app_context(fn)
         return fn
 
     def submit(self, fn, *args, **kwargs):
